@@ -1,12 +1,12 @@
 package PICA::Data;
 use v5.14.1;
 
-our $VERSION = '1.14';
+our $VERSION = '1.15';
 
 use Exporter 'import';
 our @EXPORT_OK = qw(pica_parser pica_writer pica_path pica_xml_struct
     pica_match pica_values pica_value pica_fields pica_holdings pica_items
-    pica_guess);
+    pica_guess clean_pica);
 our %EXPORT_TAGS = (all => [@EXPORT_OK]);
 
 our $ILN_PATH = PICA::Path->new('101@a');
@@ -18,6 +18,72 @@ use Encode qw(decode);
 use List::Util qw(first any);
 use IO::Handle;
 use PICA::Path;
+use PICA::Error;
+
+sub clean_pica {
+    my ($record, %options) = @_;
+
+    my $ok      = 1;
+    my $handler = $options{error};
+    my $error   = exists $options{error}
+        ? sub {
+        if ($handler) {
+            $handler->(PICA::Error->new($_[1] || [], message => $_[0]));
+        }
+        $ok = 0;
+        }
+        : sub {say STDERR shift; $ok = 0};
+
+    $record = $record->{record} if reftype $record eq 'HASH';
+
+    if (reftype $record ne 'ARRAY') {
+        $error->('PICA record must be array reference');
+    }
+    elsif (!@$record && !$options{ignore_empty_records}) {
+        $error->('PICA record should not be empty');
+    }
+
+    return unless $ok;
+
+    my @filtered;
+
+    for my $field (@$record) {
+        if (reftype $field ne 'ARRAY') {
+            $error->('PICA field must be array reference');
+            return
+        }
+
+        my ($tag, $occ, @sf) = @$field;
+
+        if ($tag !~ /^[012.][0-9.][0-9.][A-Z@.]$/) {
+            $error->("Malformed PICA tag: $tag", $field);
+        }
+
+        if ($occ) {
+            if ($occ !~ /^[0-9]{1,3}$/) {
+                $error->("Malformed occurrence: $occ", $field);
+            }
+            elsif (substr($tag, 0, 1) ne '2' && length $occ eq 3) {
+                $error->(
+                    "Three digit occurrences only allowed on PICA level 2",
+                    $field
+                );
+            }
+        }
+
+        while (@sf) {
+            my $code  = shift @sf;
+            my $value = shift @sf;
+
+            $error->("Malformed PICA subfield: $code", $field)
+                if $code !~ /^[_A-Za-z0-9]$/;
+            $error->("PICA subfield \$$code must be non-empty string", $field)
+                if $value !~ /^./;
+        }
+    }
+
+    return $record if $ok;
+}
 
 sub pica_match {
     my ($record, $path, %args) = @_;
@@ -167,6 +233,7 @@ use PICA::Writer::Plain;
 use PICA::Writer::Binary;
 use PICA::Writer::PPXML;
 use PICA::Writer::JSON;
+use PICA::Writer::Fields;
 
 sub pica_parser {
     _pica_module('PICA::Parser', @_);
@@ -488,7 +555,8 @@ expression. The following are virtually equivalent:
 
 =head2 pica_fields( $record, $path[, $path...] )
 
-Returns a PICA record (or empty array reference) limited to fields specified inione ore more PICA path expression. The following are virtually equivalent:
+Returns a PICA record (or empty array reference) limited to fields specified in
+one ore more PICA path expression. The following are virtually equivalent:
 
     pica_fields($record, $path);
     $path->record_fields($record);
@@ -504,6 +572,24 @@ accessor C<holdings>.
 Returns a list (as array reference) of item records. Also available as
 accessor C<items>.
 
+=head2 clean_pica( $record[, %options] )
+
+Returns a given PICA record as array of arrays, unless the record isn't
+syntactically valid PICA. Options include:
+
+=over
+
+=item error
+
+Error handler, prints instances of L<PICA::Error> to STDERR by default. Use
+C<undef> to ignore all errors.
+
+=item ignore_empty_records
+
+Don't emit an error if the record has no fields.
+
+=back
+
 =head1 ACCESSORS
 
 All accessors of C<PICA::Data> are also available as L</FUNCTIONS>, prefixed
