@@ -1,14 +1,15 @@
 package PICA::Schema;
 use v5.14.1;
 
-our $VERSION = '1.14';
+our $VERSION = '1.15';
 
 use Exporter 'import';
 our @EXPORT_OK = qw(field_identifier check_value);
 
 use Scalar::Util qw(reftype);
 use Storable qw(dclone);
-use PICA::Schema::Error;
+use PICA::Error;
+use PICA::Data;
 
 sub new {
     my ($class, $schema) = @_;
@@ -18,10 +19,12 @@ sub new {
 sub check {
     my ($self, $record, %options) = @_;
 
-    $record = $record->{record} if reftype $record eq 'HASH';
+    my @errors;
+    $record
+        = PICA::Data::clean_pica($record, error => sub {push @errors, shift});
+    return @errors unless $record;
 
     $options{counter} = {};
-    my @errors;
 
     my %field_identifiers;
     for my $field (@$record) {
@@ -36,7 +39,7 @@ sub check {
         my $field = $self->{fields}{$id};
         if ($field->{required} && !$field_identifiers{$id}) {
             push @errors,
-                PICA::Schema::Error->new(
+                PICA::Error->new(
                 [substr($id, 0, 4), length $id gt 4 ? substr($id, 5) : undef],
                 required => 1
                 );
@@ -68,21 +71,21 @@ sub check_field {
 
         if ($spec) {    # field is deprectaed
             unless ($opts{allow_deprecated_fields}) {
-                return PICA::Schema::Error->new($simple, deprecated => 1);
+                return PICA::Error->new($simple, deprecated => 1);
             }
         }
         elsif ($opts{ignore_unknown_fields}) {
             return ();
         }
         else {
-            return PICA::Schema::Error->new($simple);
+            return PICA::Error->new($simple);
         }
     }
 
     if ($opts{counter} && !$spec->{repeatable}) {
         my $tag_occ = join '/', grep {defined} @$field[0, 1];
         if ($opts{counter}{$tag_occ}++) {
-            return PICA::Schema::Error->new($field, repeated => 1);
+            return PICA::Error->new($field, repeated => 1);
         }
     }
 
@@ -140,9 +143,7 @@ sub check_field {
         }
     }
 
-    return %errors
-        ? PICA::Schema::Error->new($field, subfields => \%errors)
-        : ();
+    return %errors ? PICA::Error->new($field, subfields => \%errors) : ();
 }
 
 sub check_value {
@@ -251,7 +252,7 @@ Schema information can be included in PICA XML with L<PICA::Writer::XML>.
 =head2 check( $record [, %options ] )
 
 Check whether a given L<PICA::Data> record confirms to the schema and return a
-list of L<PICA::Schema::Error>. Possible options include:
+list of L<PICA::Error>. Possible options include:
 
 =over
 
@@ -296,7 +297,7 @@ Don't check subfields at all.
 =head2 check_field( $field [, %options ] )
 
 Check whether a PICA field confirms to the schema. Use same options as method
-C<check>. Returns a L<PICA::Schema::Error> on schema violation.
+C<check>. Returns a L<PICA::Error> on schema violation.
 
 =head2 abbreviated
 
@@ -312,7 +313,7 @@ field tag and optional occurrence if the tag starts with C<0>.
 =head2 check_value( $value, $schedule [, %options ] )
 
 Check a subfield value against a subfield schedule. On malformed values returns
-a L<subfield error|PICA::Schema::Error/SUBFIELD ERRORS> without C<message> key.
+a L<subfield error|PICA::Error/SUBFIELD ERRORS> without C<message> key.
 
 =head1 LIMITATIONS
 
