@@ -1,8 +1,9 @@
 package PICA::Writer::Base;
 use v5.14.1;
 
-our $VERSION = '1.14';
+our $VERSION = '1.15';
 
+use PICA::Data;
 use Scalar::Util qw(blessed openhandle reftype);
 use Term::ANSIColor;
 use Encode qw(decode);
@@ -44,25 +45,29 @@ sub write {
     $self;
 }
 
-sub write_record {
-    my ($self, $record) = @_;
-    $record = $record->{record} if reftype $record eq 'HASH';
+sub write_identifier {
+    my ($self, $field) = @_;
 
     my $fh = $self->{fh};
     my %col = %{$self->{color} // {}};
 
-    foreach my $field (@$record) {
-        $fh->print($col{tag} ? colored($field->[0], $col{tag}) : $field->[0]);
+    $fh->print($col{tag} ? colored($field->[0], $col{tag}) : $field->[0]);
 
-        if (defined $field->[1] and $field->[1] ne '') {
-            my $occ = sprintf("%02d", $field->[1]);
-            $fh->print(
-                ($col{syntax} ? colored('/', $col{syntax}) : '/')
-                . (
-                    $col{occurrence} ? colored($occ, $col{occurrence}) : $occ
-                )
-            );
-        }
+    if (defined $field->[1] and $field->[1] ne '') {
+        my $occ = sprintf("%02d", $field->[1]);
+        $fh->print(($col{syntax} ? colored('/', $col{syntax}) : '/')
+            . ($col{occurrence} ? colored($occ, $col{occurrence}) : $occ));
+    }
+}
+
+sub write_record {
+    my ($self, $record) = @_;
+    $record = PICA::Data::clean_pica($record) or return;
+
+    my $fh = $self->{fh};
+
+    foreach my $field (@$record) {
+        $self->write_identifier($field);
         $fh->print(' ');
         for (my $i = 2; $i < scalar @$field; $i += 2) {
             $self->write_subfield($field->[$i], $field->[$i + 1]);
